{--------------------------------------------------------------------------------
* Description : TscFileDrop
  By adding this small Delphi component to your form, you can control the files
  which are dropped from Windows Explorer to your form.
  Very useful when creating small tools.
* Dates : June 2001 - March 2008
* Version 1.1

* Author : Stefan Cruysberghs
* Email : stefancr@scip.be
* Website : http://www.scip.be
  Visit SCIP.be for other components, tools, articles, ...
--------------------------------------------------------------------------------
* This component is free of charge.
* The author doesn't give a warranty for error free running
  of this component and he doesn't give any support.
* Suggestions and bugs can be send by email.
--------------------------------------------------------------------------------
Installation
* Install the TscFileDrop component by adding the unit scFileDrop.pas
  to a package. Remove the DCR file in the package. Compile and Install the package.
* The TscFileDrop component can be found in the tabsheet 'SC'
  of the component palette.
--------------------------------------------------------------------------------
Version 1.0 (June 2001)
 - Initial release
Version 1.1 (March 2008)
 - Renamed Files to AcceptedFiles
 - Renamed FileCount to AcceptedFilesCount
 - Added DroppedFiles
 - Added DroppedFilesCount
--------------------------------------------------------------------------------}

unit scFileDrop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ShellApi;

type
  TscDropEvent = procedure(Sender: TObject; X, Y: Integer) of object;
  TscFileDrop = class(TComponent)
  private
    FSLAcceptedFiles : TStrings;
    FSLDroppedFiles : TStrings;
    FIntCountAccepted : Integer;
    FIntCountDropped : Integer;
    FSLFileExt : TStrings;
    FBlnEnabled : Boolean;
    FHandle : HWND;
    FDefProc : Pointer;
    FProcInstance : Pointer;
    FOnDrop : TscDropEvent;

    procedure DropFiles(HDropHandle: HDrop);
    procedure SetEnabled(Value: Boolean);
    procedure WndProc(var Msg: TMessage);
    procedure InitControl;
    procedure DestroyControl;
    procedure SetFileExt(const Value: TStrings);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    // Read-only properties
    // List with all dropped and accepted files (cleared when new files are dropped)
    property AcceptedFiles : TStrings read FSLAcceptedFiles;
    // Count dropped and accepted files
    property AcceptedFilesCount : Integer read FIntCountAccepted;
    // List with all dropped files (cleared when new files are dropped)
    property DroppedFiles : TStrings read FSLDroppedFiles;
    // Count dropped files
    property DroppedFilesCount : Integer read FIntCountDropped;

    // Enable this component
    property EnableDrop : Boolean read FBlnEnabled write SetEnabled default True;
    // List with extensions which can be excepted by application
    property AllowedFileExtensions : TStrings read FSLFileExt write SetFileExt;

    // Event occurs when files are dropped
    // Parameters are mouseposition and properties Files and FileCount are filled in
    property OnDrop : TscDropEvent read FOnDrop write FOnDrop;
  end;

procedure Register;

implementation

{$R scFileDrop.dcr}

//------------------------------------------------------------------------------
procedure register;
begin
  RegisterComponents('SC', [TscFileDrop]);
end;

//------------------------------------------------------------------------------
constructor TscFileDrop.Create( AOwner: TComponent );
begin
  inherited Create(AOwner);
  FIntCountDropped := 0;
  FIntCountAccepted := 0;
  FSLDroppedFiles := TStringList.Create;
  FSLAcceptedFiles := TStringList.Create;
  FSLFileExt := TStringList.Create;
  FHandle := 0;
  FBlnEnabled := True;
  InitControl;
  SetEnabled(FBlnEnabled);
end;

//------------------------------------------------------------------------------
destructor TscFileDrop.Destroy;
begin
  DestroyControl;
  SetEnabled(False);
  FSLAcceptedFiles.Free;
  FSLDroppedFiles.Free;
  FSLFileExt.Free;
  inherited Destroy;
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.SetEnabled(Value: Boolean);
begin
  FBlnEnabled := Value;
  // Call Win32 API to register the owner as being able to accept dropped files
  DragAcceptFiles(FHandle, FBlnEnabled);
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.SetFileExt(const Value: TStrings);
begin
  FSLFileExt.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.InitControl;
var
  WinCtrl: TWinControl;
begin
  if Owner is TWinControl then
  begin
    // Subclass the owner so this control can capture the WM_DROPFILES message
    WinCtrl := TWinControl(Owner);
    FHandle := WinCtrl.Handle;
    FProcInstance := MakeObjectInstance(WndProc);
    FDefProc := Pointer(GetWindowLong(FHandle, GWL_WNDPROC));
    SetWindowLong(FHandle, GWL_WNDPROC, Longint(FProcInstance));
  end
  else
    FBlnEnabled := False;
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.DestroyControl;
begin
  if FHandle <> 0 then
  begin
    // Restore the original window procedure
    SetWindowLong(FHandle, GWL_WNDPROC, Longint(FDefProc));
    FreeObjectInstance(FProcInstance);
  end
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.WndProc(var Msg: TMessage);
begin
  with Msg do
  begin
    // If message is drop files, process, otherwise call the original window procedure
    if Msg = WM_DROPFILES then
      DropFiles(HDrop(wParam))
    else
      Result := CallWindowProc(FDefProc, FHandle, Msg, WParam, LParam);
  end;
end;

//------------------------------------------------------------------------------
procedure TscFileDrop.DropFiles(HDropHandle: HDrop);
var
  pszFileWithPath, pszFile : PChar;
  StrFileExt : String;
  IntCountFiles : Integer;
  IntLength, IntLengthTemp : Integer;
  PointMouse : TPoint;
begin
  // Initialize
  IntLength := 128;
  pszFileWithPath := StrAlloc(IntLength);
  pszFile := StrAlloc(IntLength);
  IntCountFiles := 0;

  // Clear string lists
  FSLDroppedFiles.Clear;
  FSLAcceptedFiles.Clear;

  // Retrieve the number of files being dropped
  FIntCountDropped := DragQueryFile(HDropHandle, $FFFFFFFF, pszFile, IntLength);

  // Retrieve each file being dropped
  while (IntCountFiles < FIntCountDropped) do
  begin
    // Get  length of the filename
    IntLengthTemp := DragQueryFile(HDropHandle, IntCountFiles, nil, 0) + 1;
    // Allocate more memory when name is longer then reserved space
    if (IntLengthTemp > IntLength) then
    begin
      IntLength := IntLengthTemp;
      StrDispose(pszFileWithPath);
      pszFileWithPath := StrAlloc(IntLength);
    end;

    // Get name
    DragQueryFile(HDropHandle, IntCountFiles, pszFileWithPath, IntLength);

    // Add file to list of dropped files
    FSLDroppedFiles.Add(StrPas(pszFileWithPath));

    // Compare file extension, if allowed, then add name to list of accepted files
    StrFileExt := Copy(UpperCase(ExtractFileExt(StrPas(pszFileWithPath))),2,15);
    if (FSLFileExt.Count = 0) or (FSLFileExt.IndexOf(StrFileExt) >= 0) or (FSLFileExt.IndexOf('.'+StrFileExt) >= 0) then
      FSLAcceptedFiles.Add(StrPas(pszFileWithPath));
    Inc(IntCountFiles);
  end;

  FIntCountAccepted := FSLAcceptedFiles.Count;

  StrDispose(pszFileWithPath);

  // Called OnDrop event when files are accepted
  if Assigned(FOnDrop) and (FIntCountAccepted > 0) then
  begin
    // Get the mouse point where files were dropped
    DragQueryPoint(HDropHandle, PointMouse);
    FOnDrop(Self, PointMouse.X, PointMouse.Y);
  end;

  DragFinish(HDropHandle);
end;

end.
